//  $Id: BondFactory.java,v 1.14 2007/06/04 22:46:51 Sasha Buzko Exp $
//
//  Copyright (c) 2000-2002  San Diego Supercomputer Center (SDSC),
//  a facility operated jointly by the University of California,
//  San Diego (UCSD) and General Atomics, San Diego, California, USA.
//
//  Users and possessors of this source code are hereby granted a
//  nonexclusive, royalty-free copyright and design patent license to
//  use this code in individual software.  License is not granted for
//  commercial resale, in whole or in part, without prior written
//  permission from SDSC.  This source is provided "AS IS" without express
//  or implied warranty of any kind.
//
//  For further information, please see:  http://mbt.sdsc.edu
//
//  History:
//  $Log: BondFactory.java,v $
//  Revision 1.14  2007/06/04 22:46:51  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.13  2007/03/13 04:33:59  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.12  2007/02/13 19:08:40  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.11  2007/02/07 18:42:57  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.10  2007/02/07 02:45:49  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.9  2007/02/05 04:11:33  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.8  2006/11/21 01:02:05  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.7  2006/09/30 04:51:29  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.6  2006/08/21 01:05:10  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.5  2006/08/21 00:55:32  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.4  2006/08/16 21:36:42  Sasha Buzko
//  added gamess inp format
//
//  Revision 1.3  2006/08/02 04:13:57  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.2  2006/06/05 09:15:01  Sasha Buzko
//  *** empty log message ***
//
//  Revision 1.1  2006/05/20 17:02:03  Sasha Buzko
//  Updated version
//
//  Revision 1.1  2006/04/30 20:13:59  Sasha Buzko
//  New version of the app
//
//  Revision 1.1  2006/04/15 19:42:26  Sasha Buzko
//  Initial commit
//
//  Revision 1.5  2006/02/24 22:26:53  Administrator
//  *** empty log message ***
//
//  Revision 1.4  2006/02/20 18:03:24  Administrator
//  *** empty log message ***
//
//  Revision 1.3  2005/12/12 05:05:57  Administrator
//  *** empty log message ***
//
//  Revision 1.2  2005/11/26 04:36:11  Administrator
//  *** empty log message ***
//
//  Revision 1.1  2005/11/13 04:35:04  Administrator
//  *** empty log message ***
//
//  Revision 1.3  2003/07/11 23:08:46  moreland
//  Added generateCovalentBonds methods which use a logN Octree algorithm.
//
//  Revision 1.2  2003/04/24 21:00:42  moreland
//  Moved distance calculation code to a public method since its used a lot.
//
//  Revision 1.1  2003/04/23 22:52:43  moreland
//  Created utility methods for creating Bond objects.
//
//  Revision 1.0  2003/04/03 23:08:11  moreland
//  First version.
//


package edu.sdsc.mbt.util;


import edu.sdsc.mbt.*;

import java.util.*;


public class BondFactory
{
//  Add for internationalization
	Locale locale;
	private ResourceBundle reslang;
	reslang = ResourceBundle.getBundle("SiriusLangRes", locale);
	private String[] _M_Symbol = new String[10];
// end
	/**
	 * Cut-off distance limit for covalent bond formation.
	 */
	private static double covalentBondLimit   = 1.9;

	/**
	 * Cut-off distance limit for peptide bond formation.
	 */
	private static double peptideBondLimit    = 2.3;

	/**
	 * Cut-off distance limit for disulphide bond formation.
	 */
	private static double disulphideBondLimit = 3.0;

	/**
	 * Cut-off distance limit for hydrogen bond formation.
	 */
	private static double hydrogenBondLimit   = 4.0;
	
	
	/**
	 *  Return a vector of Bond objects extracted from a Structure
	 *  using 1.8 as the bondLimit distance.
	 *  <P>
	 */
	public static Vector generateBonds( Structure structure )
	{
		return generateBonds( structure, 1.8 );
	}

	/**
	 *  Return a vector of Bond objects extracted from a Structure
	 *  using the specified bondLimit distance.
	 *  The algorithm uses fixed-size cells to "bucket" the atom neighbors.
	 *  <P>
	 */
	public static Vector generateBonds( Structure structure, double bondLimit )
	{
		
		double coordinateBounds[][] =
			AtomStats.getAtomCoordinateBounds( structure );

		//
		// Get the atom coordinate bounds.
		//

		double minCoord[] = coordinateBounds[0];
		double maxCoord[] = coordinateBounds[1];

		//
		// Compute search cell grid dimensions
		//

		// Range of search space
		double gridRange[] = {
			maxCoord[0] - minCoord[0],
			maxCoord[1] - minCoord[1],
			maxCoord[2] - minCoord[2]
		};

		// Cell grid size
		// JLM DEBUG: A cell DIAGONAL distance should be bondLimit !?
		double cellSize[] = {
			bondLimit * 0.5,
			bondLimit * 0.5,
			bondLimit * 0.5
		};

		// Cell grid count
		int cellCount[] = {
			(int) (gridRange[0] / cellSize[0]),
			(int) (gridRange[1] / cellSize[1]),
			(int) (gridRange[2] / cellSize[2])
		};

		//
		// Build the bond search structures.
		//

		int atomCount = structure.getStructureComponentCount(
			StructureComponentRegistry.TYPE_ATOM );

		Hashtable occupiedCells = new Hashtable( atomCount );
		int cellCoord[] = new int[3];
		int cellIndex = -1;

		for ( int i=0; i<atomCount; i++ )
		{
			Atom atom = (Atom) structure.getStructureComponentByIndex(
				StructureComponentRegistry.TYPE_ATOM, i );

			// Compute the 3D cell coordinate in which the atom lives
			cellCoord[0] = (int)
				((atom.coordinate[0] - minCoord[0]) / cellSize[0]);
			cellCoord[1] = (int)
				((atom.coordinate[1] - minCoord[1]) / cellSize[1]);
			cellCoord[2] = (int)
				((atom.coordinate[2] - minCoord[2]) / cellSize[2]);

			// Map the 3D cell coordinate to a 1D cell index
			cellIndex =
				(cellCoord[0]*cellCount[1]+cellCoord[1]) * cellCount[2] + cellCoord[2];

			// Store the atom index that lives at this 1D cell index
			occupiedCells.put( new Integer( cellIndex ), new Integer( i ) );
		}

		//
		// Extract the bonds.
		//

		Vector bondList = new Vector( );

		for ( int i=0; i<atomCount; i++ )
		{
			Atom atom = (Atom) structure.getStructureComponentByIndex(
				StructureComponentRegistry.TYPE_ATOM, i );

			// Compute the cell coordinate in which the atom lives
			cellCoord[0] = (int)
				((atom.coordinate[0] - minCoord[0]) / cellSize[0]);
			cellCoord[1] = (int)
				((atom.coordinate[1] - minCoord[1]) / cellSize[1]);
			cellCoord[2] = (int)
				((atom.coordinate[2] - minCoord[2]) / cellSize[2]);

			// Check the 26 neighbors around this cell for an atom index.
			for ( int x=-1; x<=1; x++ )
			{
				for ( int y=-1; y<=1; y++ )
				{
					for ( int z=-1; z<=1; z++ )
					{
						// Skip our own cell
						if ( (x == 0) && (y == 0) && (z == 0) ) continue;

						// Map the 3D cell coordinate to a 1D cell index
						cellIndex =
							((cellCoord[0]+x)*cellCount[1]+(cellCoord[1]+y)) * cellCount[2] + (cellCoord[2]+z);

						// Try to fetch the neighbor
						Integer neighborAtom = (Integer)
							occupiedCells.get( new Integer( cellIndex ) );

						// Is there an occupied neighbor cell?
						if ( neighborAtom != null )
						{
							int i2 = neighborAtom.intValue( );
							// Only add a bond between two atoms in
							// one direction (ie: only add a bond once!)
							if ( i < i2 )
							{
								Atom atom2 = (Atom)
									structure.getStructureComponentByIndex(
									StructureComponentRegistry.TYPE_ATOM, i2 );
								// Only add a bond if atoms are close enough
								if ( distance( atom, atom2 ) <= bondLimit )
								{
									Bond bond = new Bond( atom, atom2 );
									bondList.add( bond );
								}
							}
						}
					}
				}
			}
		}

		return bondList;
	}

	/**
	 *  Given a vector of Atom objects, return a Vector of Bond objects.
	 *  The cutOffDistance parameter specifies the maximum cartesian
	 *  distance allowed to form a bond.
	 *  The algorithm uses an octree to find atom neighbors.
	 *  <P>
	 */
	public static Vector generateCovalentBonds( Vector atoms, double cutOffDistance, boolean useCovalentRestrictions )
	{
		Vector result = null;

		// Create an octree and and populate it with Atom coordinates.
		try
		{
			
			int atomCount = atoms.size( );
//			System.out.println("atomCount = " + atomCount);
			OctreeAtomItem[] octreeAtomItems = new OctreeAtomItem[ atomCount ];

			for( int i=0; i<atomCount; i++ )
			{
				Atom atom = (Atom) atoms.elementAt( i );
				octreeAtomItems[i] = new OctreeAtomItem( atom, i );
			}
			
//			System.out.println("check a");
			
			double[] margin = new double[] { 0.0, 0.0, 0.0 };
			Octree octree = new Octree( margin.length, octreeAtomItems, margin );
//			System.out.println("octree = " + octree);
			try{
				octree.build( );
			}
			catch (Exception ex){
				ex.printStackTrace();
			}

			// Generate a vector of Bond objects.
//			System.out.println("check 1");
			result = octree.getBondsVector( cutOffDistance );
//			System.out.println("result = " + result);
			
//			System.out.println("checking for H-H bonds");
			//check result for H-H bonds
			for (int i = result.size()-1; i >= 0; i--){
				Bond b = (Bond)result.get(i);
//				System.out.println("b = " + b + ", atom0 = " + b.getAtom(0).name + ", atom1 = " + b.getAtom(1).name);
				if (useCovalentRestrictions && b.getAtom(0).element == 1 && b.getAtom(1).element == 1){
					result.remove(b);
				}
			}
			
			/**
			 * TODO
			 * Take care of cases with two atoms bonded to a single H, with one of them having a normal
			 * distance, but the other being just under the cutoff (e.g., 1.88 A).
			 */
		}
		catch( OutOfMemoryError oome )
		{
// 			Status.output( Status.LEVEL_ERROR, "Not enough memory to create bonds" );
			Status.output( Status.LEVEL_ERROR, M_Notenoughmemorytoc );
			result = null;
		}
		catch( StackOverflowError soe )
		{
// 			Status.output( Status.LEVEL_ERROR, "Not enough memory to create bonds" );
			Status.output( Status.LEVEL_ERROR, M_Notenoughmemorytoc );
			result = null;
		}
		catch (Exception e){
// 			Status.output(Status.LEVEL_ERROR, "Exception generating bonds");
			Status.output(Status.LEVEL_ERROR, M_Exceptiongeneratin_1);
			e.printStackTrace();
		}

		return result;
	}
	
	
	/**
	 *  Given a vector of Atom objects, return a Vector of Bond objects.
	 *  A default bond cut-off distance of 1.9 is used.
	 *  The algorithm uses an octree to find atom neighbors.
	 *  <P>
	 */
	public static Vector generateCovalentBonds( Vector atoms )
	{
		return generateCovalentBonds( atoms, 1.9, true );
	}


	/**
	 *  Return the coordinate distance between two Atom objects.
	 *  <P>
	 */
	public static double distance( Atom atomA, Atom atomB )
	{
		double d0 = atomB.coordinate[0] - atomA.coordinate[0];
		double d1 = atomB.coordinate[1] - atomA.coordinate[1];
		double d2 = atomB.coordinate[2] - atomA.coordinate[2];
		return Math.sqrt( d0*d0 + d1*d1 + d2*d2 );
	}
}

