package edu.sdsc.sirius.search;

import edu.sdsc.mbt.util.*;
import edu.sdsc.mbt.viewables.StylesPreferences;
import edu.sdsc.mbt.*;
import edu.sdsc.sirius.dialogs.*;
import edu.sdsc.sirius.dialogs.TableImpl.*;
import edu.sdsc.sirius.io.*;
import edu.sdsc.sirius.util.Manager;
import edu.sdsc.sirius.util.DNAProperties;
import edu.sdsc.sirius.util.FileNameFilter;
import edu.sdsc.sirius.util.IOHandler;
import edu.sdsc.sirius.util.ProteinProperties;
import edu.sdsc.sirius.viewers.*;
import edu.sdsc.sirius.viewers.SequenceViewerImpl.*;
import edu.stanford.ejalbert.BrowserLauncher;
import edu.stanford.ejalbert.BrowserLauncherRunner;
import edu.stanford.ejalbert.exception.UnsupportedOperatingSystemException;

import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import uk.ac.ebi.webservices.WSInterProScan.*;

public class InterProScanServicePanel extends JPanel implements SearchServicePanel {
//  Add for internationalization
	Locale locale;
	private ResourceBundle reslang;
	reslang = ResourceBundle.getBundle("SiriusLangRes", locale);
	private String[] _M_Symbol = new String[10];
// end	
	private String name = "InterProScan";
	private Manager callable;
	
	private Entry currentEntry;
	private int currentEntryIndex = -1;//for composite entries
	private Entry activeEntry;
	
	private JPanel topPanel = new JPanel();
	private JRadioButton check1 = new JRadioButton();
	private JTextField field1 = new JTextField(100);
// 	private JRadioButton check2 = new JRadioButton("Sequence file: ");
	private JRadioButton check2 = new JRadioButton(M_Sequencefile);
	private JTextField field2 = new JTextField(100);
	
	private ButtonGroup group = new ButtonGroup();
	
	
// 	private JButton search = new JButton("Search");
	private JButton search = new JButton(M_Search_1);
// 	private JButton clear = new JButton("Clear");
	private JButton clear = new JButton(M_Clear);
// 	private JButton settings = new JButton("Settings...");
	private JButton settings = new JButton(M_Settings);
// 	private JButton browse = new JButton("Browse...");
	private JButton browse = new JButton(M_Browse);
	
	private JScrollPane scrollPane;

	
	private boolean protein = true;
	
	private String outputData;
	
	private JPanel panel1 = new JPanel();
	private JPanel panel2 = new JPanel();
	private JPanel panel3 = new JPanel();
	

	
	private JEditorPane editorPane = new JEditorPane();
		
	private JPanel buttonPanel = new JPanel();
// 	private JButton showButton = new JButton("Map to display");
	private JButton showButton = new JButton(M_Maptodisplay);
// 	private JButton webButton = new JButton("Online...");
	private JButton webButton = new JButton(M_Online);
// 	private JButton clearButton = new JButton("Clear");
	private JButton clearButton = new JButton(M_Clear);
// 	private JButton outputButton = new JButton("Save XML...");
	private JButton outputButton = new JButton(M_SaveXML);
	
	private Border border1;
	private TitledBorder border2;
	private Border border;
	
	private InterProResultPanel resultPanel;
	
	//variables for table implementation of the result list
	private Vector searchTypes;
	private DataSet set;
	private JPanel thisPanel;
	
	private volatile Thread searchRunner;
	private boolean stopSearch = false;
	
	private BrowserLauncher launcher;

	
	public InterProScanServicePanel(){
		
		try{
			launcher = new BrowserLauncher(null);
		}
		catch (UnsupportedOperatingSystemException e){
// 			callable.displayErrorMessage("Opening domain records in a browser window is not supported on your operating system");
			callable.displayErrorMessage(M_Openingdomainrecor);
		}
		catch (Exception e){
// 			callable.displayErrorMessage("Opening domain records in a browser window is disabled");
			callable.displayErrorMessage(M_Openingdomainrecor_1);
		}

		
		setLayout(new BorderLayout());

		searchTypes = new Vector();
		searchTypes.add(" ");
		searchTypes.add("Web");
		searchTypes.add("Application");
// 		searchTypes.add("Accession");
		searchTypes.add(M_Accession);
// 		searchTypes.add("Name");
		searchTypes.add(M_Name_2);
// 		searchTypes.add("Start");
		searchTypes.add(M_Start_4);
// 		searchTypes.add("End");
		searchTypes.add(M_End_1);
		searchTypes.add("E-value");
		
		editorPane.setEditable(false);
		
		thisPanel = this;

		topPanel.setPreferredSize(new Dimension(500, 135));
		topPanel.setLocation(new Point(0,0));
		topPanel.setBounds(0,0,500,135);
		topPanel.setLayout(new GridLayout(3,1));

		
		border1 = BorderFactory.createEmptyBorder(0,0,0,0);//left offset of the boxes in the box
// 		border2 = BorderFactory.createTitledBorder(new EtchedBorder(), " Search parameters ");
		border2 = BorderFactory.createTitledBorder(new EtchedBorder(), M_Searchparameters);
		border2.setTitleColor(Color.black);
// 		border2.setTitleFont(new Font("Dialog", 0, 12));
		border2.setTitleFont(new Font(M_Dialog, 0, 12));
		border = BorderFactory.createCompoundBorder(border1, border2);
		topPanel.setBorder(border);
		
		check1.setForeground(Color.black);
// 		check1.setFont(new Font("Dialog", Font.PLAIN, 12));
		check1.setFont(new Font(M_Dialog, Font.PLAIN, 12));
// 		check1.setText("Click query in a viewer: ");
		check1.setText(M_Clickqueryinaviewe);
		check1.setPreferredSize(new Dimension(150,20));
		check1.setBounds(20,25,150,20);
		check1.setLocation(new Point(20,25));
		check1.setVisible(true);
		check1.setSelected(true);
		check1.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				field2.setEditable(false);
				field2.setEnabled(false);
				field1.setEditable(true);
				field1.setEnabled(true);
			}
		});
		
		Vector types = new Vector();
// 		types.add("Protein");
		types.add(M_Protein_4);
// 		types.add("DNA");
		types.add(M_DNA);
		
		JComboBox typeCombo = new JComboBox(types);
		typeCombo.setPreferredSize(new Dimension(60,20));
		typeCombo.setBounds(10,0,60,20);
		typeCombo.setLocation(new Point(10,0));
		typeCombo.setVisible(true);
		
		JPanel temp1 = new JPanel();
		temp1.setLayout(null);
		temp1.setPreferredSize(new Dimension(70,20));
		temp1.add(typeCombo);

		field1.setPreferredSize(new Dimension(180,20));
		field1.setBounds(280,25,180,20);
		field1.setLocation(new Point(280,25));
		field1.setEditable(true);
		field1.setEnabled(true);
		field1.setVisible(true);
		field1.addKeyListener(new KeyAdapter(){
			public void keyPressed(KeyEvent k){
				if (k.getKeyCode() == KeyEvent.VK_ENTER){
					search.setEnabled(false);
// 					clear.setText("Stop");
					clear.setText(M_Stop);
					Thread runner = new Thread(){
						public void run(){
							process();
// 							clear.setText("Clear");
							clear.setText(M_Clear);
							search.setEnabled(true);
						}
					};
					runner.start();
					
				}
			}
		});

		panel1.setLayout(new BorderLayout());
		panel1.add(check1, BorderLayout.WEST);
		panel1.add(field1, BorderLayout.CENTER);
		panel1.add(temp1, BorderLayout.EAST);
		panel1.setBorder(new EmptyBorder(7,7,7,7));

		
		check2.setForeground(Color.black);
// 		check2.setFont(new Font("Dialog", Font.PLAIN, 12));
		check2.setFont(new Font(M_Dialog, Font.PLAIN, 12));
		check2.setPreferredSize(new Dimension(105,20));
		check2.setBounds(20,55,105,20);
		check2.setLocation(new Point(20,55));
		check2.setVisible(true);
		check2.setSelected(false);
		check2.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				field1.setEnabled(false);
				field1.setEditable(false);
				field2.setEnabled(true);
				field2.setEditable(true);
			}
		});
		
		field2.setPreferredSize(new Dimension(230,20));
		field2.setBounds(135,55,230,20);
		field2.setLocation(new Point(135,55));
		field2.setEditable(false);
		field2.setEnabled(false);
		field2.setVisible(true);
		field2.addKeyListener(new KeyAdapter(){
			public void keyPressed(KeyEvent k){
				if (k.getKeyCode() == KeyEvent.VK_ENTER){
					search.setEnabled(false);
// 					clear.setText("Stop");
					clear.setText(M_Stop);
					Thread runner = new Thread(){
						public void run(){
							process();
// 							clear.setText("Clear");
							clear.setText(M_Clear);
							search.setEnabled(true);
						}
					};
					runner.start();
					
				}
			}
		});
		

		browse.setPreferredSize(new Dimension(90,20));
//		browse.setLocation(new Point(380, 55));
		browse.setBounds(10, 0, 90, 20);
		browse.setVisible(true);
		browse.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				check1.setSelected(false);
				field1.setEditable(false);
				field1.setEnabled(false);
				check2.setSelected(true);
				field2.setEditable(true);
				field2.setEnabled(true);
				
				
				JFileChooser chooser = new JFileChooser(StylesPreferences.workingDirectory);
				chooser.setApproveButtonText("OK");
				chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
// 				chooser.addChoosableFileFilter( new FileNameFilter("fst", "FASTA format") );
				chooser.addChoosableFileFilter( new FileNameFilter("fst", M_FASTAformat) );
// 				chooser.addChoosableFileFilter( new FileNameFilter("aln", "ALN format") );
				chooser.addChoosableFileFilter( new FileNameFilter("aln", M_ALNformat) );
			
				int result = chooser.showOpenDialog(callable.getApplicationFrame());
				if (result == JFileChooser.CANCEL_OPTION){
					return;
				}
				File filename = chooser.getSelectedFile();
				field2.setText(filename.toString());
				currentEntry = null;
			}
		});

		panel2.setLayout(new BorderLayout());
		panel2.add(check2, BorderLayout.WEST);
		panel2.add(field2, BorderLayout.CENTER);
		
		JPanel temp = new JPanel();
		temp.setLayout(null);
		temp.setPreferredSize(new Dimension(100,20));
		temp.add(browse);
		panel2.add(temp, BorderLayout.EAST);
		panel2.setBorder(new EmptyBorder(7,7,7,7));
		
		
		
		search.setPreferredSize(new Dimension(70,25));
		search.setLocation(new Point(0, 0));
		search.setBounds(0, 0, 70, 25);
		search.setVisible(true);
		search.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				search.setEnabled(false);
// 				clear.setText("Stop");
				clear.setText(M_Stop);
				settings.setEnabled(false);
				try{
					searchRunner = new Thread(){
						public void run(){
							process();
// 							clear.setText("Clear");
							clear.setText(M_Clear);
							search.setEnabled(true);
							settings.setEnabled(true);
						}
					};
					searchRunner.start();
				}
				catch (Exception e){}
			}
		});
		
		clear.setPreferredSize(new Dimension(60,25));
		clear.setLocation(new Point(80, 0));
		clear.setBounds(80, 0, 60, 25);
		clear.setVisible(true);
		clear.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
// 				if (clear.getText().equals("Stop")){
				if (clear.getText().equals(M_Stop)){
					//search is in progress, cancel it
					
					if (searchRunner.isAlive()){
						stopSearch = true;
						
// 						clear.setText("Clear");
						clear.setText(M_Clear);
						search.setEnabled(true);
						settings.setEnabled(true);
					}
					return;
				}
				
				
				field1.setText("");
				field2.setText("");
			}
		});
		
		settings.setPreferredSize(new Dimension(90,25));
		settings.setLocation(new Point(150, 0));
		settings.setBounds(150, 0, 90, 25);
		settings.setVisible(true);
		settings.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				//check current settings
				//get display dialog status first
				boolean status = callable.getDisplayDialogStatus();
				if (status) return;
				Thread runner = new Thread(){
					public void run(){
						InterProDialog d = new InterProDialog(callable.getApplicationFrame(), callable);
					}
				};
				runner.start();
			}
		});
		
		group.add(check1);
		group.add(check2);
		
		JPanel panel3a = new JPanel();
		panel3a.setLayout(null);
		panel3a.setPreferredSize(new Dimension(250, 30));
		panel3a.add(search);
		panel3a.add(clear);
		panel3a.add(settings);
		
		JPanel panel3b = new JPanel();
		panel3b.setLayout(null);
		panel3b.setPreferredSize(new Dimension(115, 30));
		
		panel3.setLayout(new BorderLayout());
		panel3.add(panel3a, BorderLayout.WEST);
		panel3.add(panel3b, BorderLayout.EAST);
		panel3.setBorder(new EmptyBorder(3,3,3,7));
		
		
		topPanel.add(panel1);
		topPanel.add(panel2);
		topPanel.add(panel3);
		
//		editorPane.setEditable(false);
		scrollPane = new JScrollPane(editorPane);
		
		buttonPanel.setPreferredSize(new Dimension(500, 45));
//		buttonPanel.setLocation(new Point(0,0));
//		buttonPanel.setBounds(0,0,500,100);
		buttonPanel.setLayout(null);
		
		
		showButton.setPreferredSize(new Dimension(110,25));
		showButton.setLocation(new Point(5, 10));
		showButton.setBounds(5, 10, 110, 25);
		showButton.setVisible(true);
		showButton.setEnabled(false);
		showButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				showMapping();
			}
		});

		webButton.setPreferredSize(new Dimension(90,25));
		webButton.setLocation(new Point(120, 10));
		webButton.setBounds(120, 10, 90, 25);
		webButton.setVisible(true);
		webButton.setEnabled(false);
		webButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				showOnline();
			}
		});

		clearButton.setPreferredSize(new Dimension(70,25));
		clearButton.setLocation(new Point(215, 10));
		clearButton.setBounds(215, 10, 70, 25);
		clearButton.setVisible(true);
		clearButton.setEnabled(false);
		clearButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
		        thisPanel.remove(resultPanel);
		        resultPanel = null;
		        outputData = null;
		        set = null;
		        thisPanel.add(scrollPane, BorderLayout.CENTER);

		        revalidate();
		        repaint();
//				printButton.setEnabled(false);
				clearButton.setEnabled(false);
				outputButton.setEnabled(false);
				showButton.setEnabled(false);
				webButton.setEnabled(false);
			}
		});

		outputButton.setPreferredSize(new Dimension(100,25));
		outputButton.setLocation(new Point(290, 10));
		outputButton.setBounds(290, 10, 100, 25);
		outputButton.setVisible(true);
		outputButton.setEnabled(false);
		outputButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (outputData != null && outputData.length() > 0){
					//save it to a file
					IOHandler.saveTextFile(outputData, callable, "xml");
				}

			}
		});

		
		buttonPanel.add(showButton);
		buttonPanel.add(webButton);
		buttonPanel.add(clearButton);
		buttonPanel.add(outputButton);
		
		this.add(topPanel, BorderLayout.NORTH);
		this.add(scrollPane, BorderLayout.CENTER);
		this.add(buttonPanel, BorderLayout.SOUTH);
		
	}
	
	private void process(){
		
		try{
			
			//get the sequence
			String sequence = null;
			
			activeEntry = currentEntry;//to keep Entry used in search separate from any random clicks, once the results are displayed
			InputParams params = new InputParams();
			if (check1.isSelected()){
				if (currentEntry == null){
// 					callable.displayErrorMessage("Nothing has been selected: No query sequence");
					callable.displayErrorMessage(M_Nothinghasbeensele);
					return;
				}
				
				
				if (currentEntry instanceof StructureEntry){
//					System.out.println("structure entry");
					//derive a sequence from the structure
					StringBuffer buffer = new StringBuffer();
					Structure structure = ((StructureEntry)currentEntry).getStructure();
					StructureMap map = structure.getStructureMap();
					for (int i = 0; i < map.getResidueCount(); i++){
						Residue r = map.getResidue(i);
						if (ProteinProperties.isAminoAcid(r.getCompoundCode())){
							buffer.append(AminoAcidInfo.getLetterFromCode(r.getCompoundCode()));
						}
						else if (DNAProperties.isNucleicAcid(r.getCompoundCode())){
							buffer.append(r.getCompoundCode());//it's already single letter
						}
					}
					
					sequence = buffer.toString();
				}
				else if (currentEntry instanceof SequenceEntry){
//					System.out.println("sequence entry");
					//get the sequence directly
					String temp = null;
					if (currentEntryIndex == -1){
						temp = ((SequenceEntry)currentEntry).getSequence().getSequence();
					}
					else{
						temp = ((SequenceEntry)currentEntry).getSequence(currentEntryIndex).getSequence();
					}
					
					Pattern pattern = Pattern.compile("-");
					Matcher matcher = pattern.matcher(temp);
					String temp2 = matcher.replaceAll("");

					Pattern pattern2 = Pattern.compile("\\*");
					Matcher matcher2 = pattern.matcher(temp2);
					sequence = matcher.replaceAll("");
				}
			}
			else{
				//file option
				
				if (field2.getText().length() == 0){
// 					callable.displayErrorMessage("Nothing has been selected: No query sequence");
					callable.displayErrorMessage(M_Nothinghasbeensele);
					return;
				}
				
				String filename = field2.getText();
				Entry e = null;
				if (filename.endsWith("aln")){
					e = (new AlnEntryLoader()).load(filename, -1);
				}
				else if (filename.endsWith("fst") || filename.endsWith("fasta")){
					e = (new FastaEntryLoader()).load(filename, -1);
				}
				
				if (e == null){
// 					callable.displayErrorMessage("Unable to load selected file. Please try again.");
					callable.displayErrorMessage(M_Unabletoloadselect);
					return;
				}
				
				sequence = ((SequenceEntry)e).getSequence(0).getSequence();

			}
			
//			System.out.println("sequence = " + sequence);
			
			
			if (sequence == null || sequence.length() == 0){
// 				callable.displayErrorMessage("Query sequence is empty. Please check query selection and try again.");
				callable.displayErrorMessage(M_Querysequenceisemp);
				return;
			}
			
			//clean up the previous stuff
			if (resultPanel != null){
		        thisPanel.remove(resultPanel);
		        thisPanel.add(scrollPane, BorderLayout.CENTER);
		        resultPanel = null;
		        
		        showButton.setEnabled(false);
		        clearButton.setEnabled(false);
		        outputButton.setEnabled(false);
		        
		        
		        revalidate();
		        repaint();
			}

        	set = null;
			
			
			
			
			Data[] inputs = new Data[1];
			Data input= new Data();
// 			input.setType("sequence");
			input.setType(M_sequence_1);
			input.setContent(sequence);
			inputs[0]=input;
			
			//set parameters
			params.setOutformat("txt");
			params.setAsync(false);
			params.setEmail("obuzko@sdsc.edu");
			
			if (protein){
// 				params.setSeqtype("protein");
				params.setSeqtype(M_protein);
			}
			else{
// 				params.setSeqtype("DNA");
				params.setSeqtype(M_DNA);
			}
			
			params.setTrlen(InterProSettings.trlen);
			
			/**
			 * TODO for DNA, figure out where to get the value for translation table
			 */
			params.setTrtable(2);
			
			params.setCrc(true);
			params.setGoterms(true);
			
			params.setApp(InterProSettings.getApps());
			params.setTrlen(InterProSettings.trlen);

		    WSInterProScanService service =  new WSInterProScanServiceLocator();
		    WSInterProScan iprscan = service.getWSInterProScan();
		    String jobid = iprscan.runInterProScan(params,inputs);
		    
		    if (stopSearch){
		    	stopSearch = false;
		    	return;
		    }
	
		    String[] results = getResults(jobid);
		    
		    if (stopSearch){
		    	stopSearch = false;
		    	return;
		    }
		    
		    //each item is the complete result for one blast search
		    //just take the first element
		    String output = results[0];
		    if (output == null || output.length() == 0){
// 		    	callable.displayErrorMessage("InterProScan search produced no results.");
		    	callable.displayErrorMessage(M_InterProScansearch);
		    	return;
		    }
		    
		    outputData = results[1];
		    if (outputData != null && outputData.length() > 0){
		    	outputButton.setEnabled(true);
		    }
		    
		    //parse the output
//		    System.out.println(output);

		    set = new DataSet();
		    
		    //parse header
		    StringTokenizer tok = new StringTokenizer(output, "\n", false);
		    int counter = 1;
		    while (tok.hasMoreTokens()){
		    	String token = tok.nextToken();

		    	String[] line = new String[6];
		    	int j = 0;
		    	
		    	//parse this token using tabs as delimiters
		    	//skip the first two fields
//		    	System.out.println("token = " + token);
		    	int index1 = token.indexOf("\t");
		    	String next = token.substring(index1+1);
//		    	System.out.println("next = " + next);
		    	int index2 = next.indexOf("\t");
		    	String next2 = next.substring(index2+1);
//		    	System.out.println("next2 = " + next2);
		    	int index3 = next2.indexOf("\t");
		    	String feed = next2.substring(index3+1);
//		    	System.out.println("feed = " + feed);
		    	StringTokenizer tt = new StringTokenizer(feed,"\t", false);
		    	while (tt.hasMoreTokens()){
		    		String t = tt.nextToken();
		    		line[j] = t;
		    		if (j == 5) break;
		    		j++;
		    	}
		    	
		    	InterProScanResultRecord record = new InterProScanResultRecord();
		    	record.application = line[0];
		    	record.accession = line[1];
		    	record.name = line[2];
		    	record.start = line[3];
		    	record.end = line[4];
		    	record.evalue = line[5];
		    	
		    	InterProDataBlock block = new InterProDataBlock();
		    	block.id = new Integer(counter).toString();
		    	
	    		block.setName(record.name);
	    		block.setAcc(record.accession);
	    		block.setApplication(record.application);
	    		block.setEv(record.evalue);
	    		block.setStart(record.start);
	    		block.setEnd(record.end);
	    		
	    		set.addData(block);
	    		counter++;
		    }
		    
		    if (set.size() > 0){
		    	if (activeEntry != null) showButton.setEnabled(true);
		    	webButton.setEnabled(true);
		    	clearButton.setEnabled(true);
		    	outputButton.setEnabled(true);
		    }
		    
	        resultPanel = new InterProResultPanel(this, set);
	        this.remove(scrollPane);
	        this.add(resultPanel, BorderLayout.CENTER);

	        revalidate();
	        repaint();
	        
	        
		}
		catch (Exception ex){
// 			callable.displayExceptionMessage("Unable to run InterProScan search", ex);
			callable.displayExceptionMessage(M_UnabletorunInterPr, ex);
			search.setEnabled(true);
			settings.setEnabled(true);
// 			clear.setText("Clear");
			clear.setText(M_Clear);
        	clearButton.setEnabled(false);
        	outputButton.setEnabled(false);
        	
		}
	}
	
	public String getSearchServiceName(){
		return name;
	}
	
	public void setParentApplication(Manager c){
		callable = c;
	}
	
	/**
	 * A temporary solution for now: index denotes a specific sequence within a composite entry
	 * @param entry
	 * @param index
	 */
	public void processClick(StructureComponent component, Entry entry, int index){
		//get the name from the entry
		
		/**
		 * TODO later add a more complicated analysis, to look for descriptive lines in a parsed pdb file, etc.
		 */
		this.currentEntry = entry;
		this.currentEntryIndex = index;
		if (entry == null){
			field1.setText("");
		}
		else{
			if (entry instanceof SequenceEntry){
				if (index != -1){
					Sequence s = ((SequenceEntry)entry).getSequence(index);
					field1.setText(s.getName());
				}
				else{
					Sequence s = ((SequenceEntry)entry).getSequence();
					field1.setText(s.getName());
				}
			}
			else{
				field1.setText(entry.getName());
			}
			
		}
		field1.setEnabled(true);
		check1.setSelected(true);
		check2.setSelected(false);
		field1.setEditable(true);
		field2.setEnabled(false);
		field2.setEditable(false);
	}
	
	public void setFocus(){
		field1.requestFocus();
	}
	
	private static String[] getResults(String jobid) throws IOException {
		try {
		    WSInterProScanService service =  new WSInterProScanServiceLocator();
		    WSInterProScan iprscan = service.getWSInterProScan();
		    WSFile[] results = iprscan.getResults(jobid);
				
			String[] out = new String[results.length];
		    for (int i=0;i<results.length;i++){
				WSFile file = results[i];
				byte[] resultbytes = iprscan.poll(jobid,file.getType());
				String result = new String(resultbytes);
				out[i] = result;
		    }
			
			return out;
		}
		catch (Exception e) {
// 			System.out.println("ERROR:\n" + e.toString());
			System.out.println(M_ERROR+"\n" + e.toString());
			e.printStackTrace();
		}
		
		return null;
	}
	
	
	public final Entry getCurrentEntry() {
		return currentEntry;
	}

	public final void setCurrentEntry(Entry currentEntry) {
		this.currentEntry = currentEntry;
		field1.setText("");
	}

	private void showMapping(){
		
		if (activeEntry == null) return;

		if (resultPanel != null){
			
			if (resultPanel.getSelectedRows() == null) return;
			
			if (resultPanel.getSelectedRowCount() == 0){
// 				callable.displayMessage("Nothing is selected. Choose a citation and try again");
				callable.displayMessage(M_NothingisselectedC_1);
				return;
			}
			
			int index = resultPanel.getSelectedRows()[0];
			InterProDataBlock block = (InterProDataBlock)resultPanel.getDataBlock(index);
			if (block == null) return;
			try{
				//use the current entry and start and end tags in the block
				int start = Integer.parseInt(block.getStart());
				int end = Integer.parseInt(block.getEnd());
				if (activeEntry.TYPE == Entry.STRUCTURE_ENTRY){
					Structure structure = ((StructureEntry)activeEntry).getStructure();
					StructureMap map = structure.getStructureMap();
					//clear existing selection
					map.getStructureStyles().selectNone(true, true);
					Vector chains = new Vector();
					int position = 0;
					for (int i = 0; i < map.getResidueCount(); i++){
						position = i + 1;//position is 1-based
						if (position >= start && position <= end){
							//select this residue
							Residue r = map.getResidue(i);
							map.getStructureStyles().setSelected(map.getResidue(i), true, true, true);
							Chain c = map.getChain(r.getAtom(0));
							if (!chains.contains(c)) chains.add(c);
						}
					}
					for (int i = 0; i < chains.size(); i++){
						map.getStructureStyles().updateChain((Chain)chains.get(i), true);
//						map.getStructureStyles().registerChainSelection((Chain)chains.get(i), true);
					}
					
					callable.updateView();
				}
				else if (activeEntry.TYPE == Entry.SEQUENCE_ENTRY){
					//there is no associated structure, so just select a range of residues
					Sequence sequence = ((SequenceEntry)activeEntry).getSequence();
					SequenceViewer sv = callable.getSequenceViewer();
					
					//get the index of this Sequence in sv
					int j = sv.getSequenceObjects().indexOf(sequence);
					System.out.println("index = " + j);
					sv.clearCellSelection(false);
					Cell[] cells = (Cell[])sv.getSequences().get(j);
					Vector selection = new Vector();
					int position = 0;
					for (int i = 0; i < cells.length; i++){
						position = i + 1;
						if (position >= start && position <= end){
							System.out.println("added cell at " + position);
							selection.add(cells[i]);
						}
					}
					
					System.out.println("size of selection = " + selection);
					sv.selectCells(sequence, selection);
				}
				

			}
			catch (Exception ex){
// 				callable.displayExceptionMessage("Unable to map the region to the loaded entry", ex);
				callable.displayExceptionMessage(M_Unabletomaptheregi, ex);
				return;
			}
			
		}
	
	}
	
	private void showOnline(){

		if (resultPanel != null){
			if (resultPanel.getSelectedRows() == null) return;
			
			if (resultPanel.getSelectedRowCount() == 0){
// 				callable.displayMessage("Nothing is selected. Choose a domain hit and try again");
				callable.displayMessage(M_NothingisselectedC_2);
				return;
			}
			
			int index = resultPanel.getSelectedRows()[0];
			InterProDataBlock block = (InterProDataBlock)resultPanel.getDataBlock(index);
			if (block == null) return;
			try{
				//use the current entry and source and accession information to launch a browser window
				String source = block.getApplication();
				
				String base = (String)InterProSettings.url.get(source);
				
				if (base == null){
// 					callable.displayErrorMessage("Unable to find the record online");
					callable.displayErrorMessage(M_Unabletofindtherec);
					return;
				}
				
				String url = base + block.getAcc();
				
				//open a web browser
				BrowserLauncherRunner runner = new BrowserLauncherRunner(launcher, url, null);
				Thread launcherThread = new Thread(runner);
				launcherThread.start();


			}
			catch (Exception ex){
// 				callable.displayExceptionMessage("Unable to display source information.", ex);
				callable.displayExceptionMessage(M_Unabletodisplaysou, ex);
				return;
			}
			
		}
	
	}
	
}
